/*
** This pi calculation program implements the Salamin / Brent / Gauss
** Arithmetic-Geometric Mean pi formula.
**
** Version: v1.5, July 24, 1998.
**
**

This  program  and documentation is Copyrighted by its author, Carey
Bloodworth, on July 18, 1998.   You  may  do what ever you wish with
them, with two exceptions.   First,  you  have  to leave my name and
this notice on it, and second, you may  not  put  the  infamous  GNU
'copyleft'  on  it.   The  GNU 'copyleft' is far too restrictive and
quite 'militant' in its wording.  I don't approve of it and I do not
want my  code  being  inflicted  by  it.   Beyond  that,  you can do
whatever you want with it.  You may use it in any  application,  for
any reason, without informing me.

Carey Bloodworth

**
**
** The main reason it's not in the Public Domain is  because  I  put
** quite a bit of work  into  these  and  I'm rather  proud  of  the
** results, and I just want my name left in. (I guess you could call
** it  pride, or  perhaps  ego.)   Beyond  that, you can do whatever
** you want with them.   You  can  use  them  for  any purpose.  You
** don't have to ask me for permission or notify  me  that  you  are
** using it.
**
** I'd also appreciate it if you see any ideas in my program and
** implement them in yours, that you put a note in it about it.
** You don't have to, though.  It's just nice to see your own name
** in somebody's list of 'thank you's.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <time.h>
#include <limits.h>
#include <float.h>
#include <ctype.h>
#include "pi.h"

Short *DSWork=NULL, *MulProd=NULL, *OldRoot=NULL;
int FFTArraySize=0;
double MaxFFTError = 0.0;
double *FFTNum1=NULL, *FFTNum2=NULL;
int FFTCache=0;
int FFTCmd=0;

/*
** DSWork is explicitly used in Divide() and Sqrt()
**   DSWork stands for _D_ivide() & _S_qrt() _Work_ var.
** OldRoot is used in the Sqrt() to hold the previous reciprocal
**   square root.  This saves us some time in Sqrt().
** MulProd is where the multiplication puts its double
**   length answer.  It's used in lots of places, of course.
**   It's also big enough for all of FractalMul()'s needs.
** MaxFFTError is how close the FFT itself is to failing.
** FFTNum1 and FFTNum2 are the two 'massive' arrays used in the
**   Fast Fourier Transform used to multiply the numbers.
** FFTArraySize is the number of bytes consumed by the FFT arrays.
**   Just for convenience.
** FFTCache tells the FFT whether an argument has already been
**   transformed and is 'cached'.
**   0 means it is not cached and you should do the transform.
**   1 means arg1 is cached and you don't need to do the transform.
**   Only the first argument is automatically saved/cached.
**   Squares are not cached unless FFTCmd is set.
**   The transform space for the second argument is used as the
**   untransform space and will destroy the arg2 transform.
**   The variable is flagged manually.  It only effects numbers
**   that are fully done by the FFTMul.  If the FractalMul or
**   the slow, regular mul is done, the cache will be disabled.
** FFTCmd is a special command to the FFT.
**   Nonzero means to save the transform of the second argument
**   instead of the first.  This is also used with squaring, since
**   the FFTMul does squaring without bothering the already cached
**   number (ie: it doesn't save it), and the only way to save the
**   number is explicitly tell it to do so.
*/

int name;
char filename[100], yon[100], noy[100], formula[100];
clock_t LoopTime, NextLoopTime, StartTime, EndTime;

void MakeName(void) {
	#if defined IS_WINDOWS
		sprintf(filename, "%d Digits of Pi.TXT", name);
	#elif defined IS_UNIX
		sprintf(filename, "%d_digits_of_pi", name);
	#else
		sprintf(filename, "%d Digits of Pi", name);
	#endif
}

/*
** A fatal error of some sort occured.  Print out the info and
** abort the program.
*/
void FatalError(char *fmt,...) {
	va_list argptr;
	va_start(argptr, fmt);
	vfprintf(stderr, fmt, argptr);
	fflush(stderr);
	va_end(argptr);
	EXIT
}

/*
***************************************
**      Input / Output routines      **
***************************************
*/

/*
** Print out a simple straight representation of our big numbers
** Used only in debugging sessions.
*/
void PrintShort2(char *Str, Short *Num, int Len) {
	int x;

	printf("%10.10s : ", Str);
	printf("%u.", Num[0]/1000);
	printf("%03u",Num[0]%1000);
	for (x = 1; x < Len/2; x++)
		printf("%04u", Num[x]);
	printf(":");
	for (x = Len/2; x < Len; x++)
		printf("%04u", Num[x]);
	printf("\n");
}

/*
** Print out a formated representation of our big numbers
**
** The formatting is based on how the Gutenberg PIMIL10.TXT
** is formatted, allowing easy comparision of results.
*/

static void PrintDig(int Dig,long printed) {
	/* Local function for PrintShort() */
	printf("%d",Dig);
	if ((printed % 1000) == 0) printf("\n\n\n\n");
	else if ((printed % 50) == 0) printf("\n");
	else if ((printed % 10) == 0) printf(" ");
}

void PrintShort(char *Str, Short *Num, int Len) {
	int x;
	int printed = 0;

	printf("%s", Str);

	printf("%u.\n\n", Num[0]/1000);
	PrintDig((Num[0]/100)%10,++printed);
	PrintDig((Num[0]/10)%10,++printed);
	PrintDig(Num[0]%10,++printed);

	for (x = 1; x < Len; x++) {
		if (Num[x]<0) printf("-");
		if (Num[x]>=10000) printf("+");
		PrintDig((Num[x]/1000)%10,++printed);
		PrintDig((Num[x]/100)%10,++printed);
		PrintDig((Num[x]/10)%10,++printed);
		PrintDig(Num[x]%10,++printed);
	}
	printf("\n");
}

/*
*******************************
**      Low level stuff      **
*******************************
*/

/*
** The lower of two int numbers
*/
int Min(int Num1, int Num2) {
	if (Num1 < Num2) return Num1;
	return Num2;
}

/*
** a=b
**
** Numbers can overlap.  You normally aren't going to do that, but
** in case you do (such as if you are experimenting with a new
** formula etc.), it'll do it correctly.
*/
void Copy(Short *a, Short *b, int Len) {
	memmove(a,b,Len*sizeof(Short));
}

/*
** Find where two numbers differ.  Useful for checking convergence.
*/
int DiffWhere(Short *Num1, Short *Num2, int Len) {
	int x;

	for (x = 0; x < Len; x++)
		if (Num1[x] != Num2[x]) return x;
	return Len;
}

/*
** Is 'a' exactly zero?
*/
int IsZero(Short *Num, int Len) {
	while (Len--)
		if (*Num++ != 0) return 0;
	return 1;
}

/*
** Is number not equal to zero?
*/
int IsNotZero(Short *Num, int Len) {
	int x;
	for (x = 0; x < Len; x++)
		if (*Num++ != 0) return x+1;
	return 0;
}

/*
** Count the number of zeros in the number.  Knowing that can let
** us do a special type of multiplication, which is faster than
** the normal one.
*/
int CountZeros(Short *Num, int Len) {
	int count;
	count=0;
	while (Len) if (Num[--Len]==0) count++;
	return count;
}

/*
** n = n / 2
*/
void DivBy2(Short * Num, int Len) {
	int x;
	unsigned long temp;

	temp = 0;
	for (x = 0; x < Len; x++) {
		temp = ((unsigned long)Num[x]) + temp * 10000;
		Num[x] = temp / 2;
		temp = temp % 2;
	}
}

/*
** n = n / 10
*/
void DivBy10(Short * Num, int Len) {
	int x;
	unsigned long temp;

	temp = 0;
	for (x = 0; x < Len; x++) {
		temp = ((unsigned long)Num[x]) + temp * 10000;
		Num[x] = temp / 10;
		temp = temp % 10;
	}
}

/*
** Num=Num*dig
**
** Simple multiplication of 'Num' by any value.  It doesn't do
** shift or overflow, etc.  It's just a simple, basic single
** value multiplication routine.  It trusts you to know when
** to properly use it.
**
** It's a bit slow (since it uses floating point), so it's not
** something you should use very often.  It's only really useful
** in the places where we need to multiply by a power of two.
*/
void MulByFloat(Short *Num, double By, int Len) {
	int x;
	double Carry,Temp;

	Carry = 0;
	for (x = Len-1; x >= 0; x--) {
		Temp = Num[x]*By + Carry;
		Carry = floor(Temp / 10000.0);
		Num[x] = Temp - Carry * 10000.0;
	}
}

/*
** Num=Num*dig
**
** Simple multiplication of 'Num' by any value.  It doesn't do
** shift or overflow, etc.  It's just a simple, basic single
** value multiplication routine.  It trusts you to know when
** to properly use it.
*/
void MulBy(Short *Num, int By, int Len) {
	int x;
	Long Temp;

	Temp = 0;
	for (x = Len-1; x >= 0; x--) {
		Temp = (Long)Num[x]*By + Temp;
		Num[x] = (Temp % 10000UL);
		Temp = (Temp / 10000UL);
	}
}

/*
** n = n * 10
*/
void MulBy10(Short *Num, int Len) {
	int x;
	unsigned long temp;

	temp = 0;
	for (x = Len-1; x >= 0; x--) {
		temp = ((unsigned long)Num[x])*10 + temp;
		Num[x] = (temp % 10000UL);
		temp   = (temp / 10000UL);
	}
}


/*int IsPow2(int Num) {
	return ((Num & -Num) == Num);
}*/

/*
** This copies a 2x mul result into a normal result variable.
** Our number format will never have more than one unit of integer,
** and after a mul, we have two, so we need to fix that.
**
** a=(b*10)
*/
void CopyProd(Short * a, Short * b, int Len) {
	MulBy10(b,Len);
	Copy(a,b,Len);
}



/*
*******************************
**         Basic math        **
*******************************
*/

/*
** sum = a + b
** Just like a regular add, except the carry can ripple on up higher,
** beyond where we end our addition.
*/
Short RippleAdd(Short * limit, Short * sum, Short * a, Short * b, int Len) {
	Short temp, carry;

	carry = 0;
	a += Len;
	b += Len;
	sum += Len;
	while (Len--) {
		temp = *(--a) + *(--b) + carry;
		*(--sum) = temp % 10000;
		carry = temp / 10000;
	}

	while ((sum > limit) && (carry != 0)) {
		temp = *(--sum) + carry;
		*sum = temp % 10000;
		carry = temp / 10000;
	}
	return carry;
}

/*
** sum = a + b
*/
Short Add(Short * sum, Short * a, Short * b, int Len) {
	return RippleAdd(sum, sum, a, b, Len);
}

/*
** dif = a-b
** Just like a regular sub, except the borrow can ripple on up higher,
** beyond where we end our subtraction.
*/
Short RippleSub(Short * limit, Short * dif, Short * a, Short * b, int Len) {
	Short temp, borrow;

	borrow = 0;
	a += Len;
	b += Len;
	dif += Len;
	while (Len--) {
		temp = *(--a) - *(--b) - borrow;
		borrow = 0;
		if (temp < 0) {
			borrow = 1;
			temp += 10000;
		}
		*(--dif) = temp;
	}

	while ((dif > limit) && (borrow != 0)) {
		temp = *(--dif) - borrow;
		borrow = 0;
		if (temp < 0) {
			borrow = 1;
			temp += 10000;
		}
	*dif = temp;
	}
	return borrow;
}

/*
** dif = a-b
*/
Short Sub(Short * dif, Short * a, Short * b, int Len) {
	return RippleSub(dif, dif, a, b, Len);
}

/*
** num = 0 - num
*/
void Negate(Short * num, int Len) {
	int x;
	Short d, borrow;

	borrow = 0;
	for (x = Len - 1; x >= 0; x--) {
		d = 0 - num[x] - borrow;
		borrow = 0;
		if (d < 0) {
			borrow = 1;
			d += 10000;
		}
	num[x] = d;
	}
}

/*
*******************************
**       Multiplication      **
*******************************
*/

/*
** Prod = Num1 * Num2
** Simple, slow multiplication method.  The space for 'Prod'
** should of course be double length.  It is most efficient
** when Num2 is mostly zeros.
*/
void SlowMul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	Long p;
	int Ndx1, Ndx2, PNdx;

	for (PNdx = 0; PNdx < Len * 2; PNdx++)
		Prod[PNdx] = 0;
	for (Ndx2 = Len - 1; Ndx2 >= 0; Ndx2--) {
		if (Num2[Ndx2] == 0)
			continue;
		PNdx = Ndx2 + Len;
		p = 0;
		for (Ndx1 = Len - 1; Ndx1 >= 0; Ndx1--) {
			p = (Long) Num1[Ndx1] * (Long) Num2[Ndx2] + p + (Long) Prod[PNdx];
			Prod[PNdx] = (Short) (p % 10000);
			p = p / 10000;
			PNdx--;
		}
		while ((p) && (PNdx >= 0)) {
			p += (Long) Prod[PNdx];
			Prod[PNdx] = (Short) (p % 10000);
			p = p / 10000;
			PNdx--;
		}
	}
}

/*
** A simple digit by digit multiplication.
**
** Like SlowMul(), except this one doesn't clear its destination
** first, and lets the carries propogate up for 'ever'.  This way,
** we can use it in our 'SmartMul()'.
*/
void AddSlowMul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	Long p;
	int Ndx1, Ndx2, PNdx;

	for (Ndx2 = Len - 1; Ndx2 >= 0; Ndx2--) {
		if (Num2[Ndx2] == 0)  continue;
		PNdx = Ndx2 + Len;
		p = 0;
		for (Ndx1 = Len - 1; Ndx1 >= 0; Ndx1--) {
			p = (Long) Num1[Ndx1] * (Long) Num2[Ndx2] + p + (Long) Prod[PNdx];
			Prod[PNdx] = (Short) (p % 10000);
			p = p / 10000;
			PNdx--;
		}
		while (p) {
			p += (Long) Prod[PNdx];
			Prod[PNdx] = (Short) (p % 10000);
			p = p / 10000;
			PNdx--;
		}
	}
}

/*
** This is based on the simple Fractal / Divide and Conquer / Karatsuba
** O(n^1.585) method.
**
** It's fairly simple.  a*b is: a1b1(B^2+B)+(a1-a2)(b2-b1)B+a2b2(B+1)
**
** You need 4*SIZE storage for the product and the working space.
*/
void FractalMul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	int x, HLen, Sign1, Sign2;
	Short *offset;
	Short *Work=Prod+2*Len;

	HLen = Len/2;

	/* Do x=Right(Num1)-Left(Num1) y=Left(Num2)-Right(Num2) */
	Sign1 = Sub(Prod, Num1+HLen, Num1, HLen);     if (Sign1) Negate(Prod, HLen);
	Sign2 = Sub(Prod+HLen, Num2, Num2+HLen, HLen);if (Sign2) Negate(Prod+HLen, HLen);

	Mul(Work, Prod, Prod+HLen, HLen);

	for (x=0;x<Len*2;x++) Prod[x]=0;
	offset = Prod + HLen;
	if (Sign1 == Sign2) RippleAdd(Prod,offset,offset,Work,Len);
	else RippleSub(Prod,offset,offset,Work,Len);

	Mul(Work, Num1, Num2, HLen);
	offset = Prod + HLen;RippleAdd(Prod,offset,offset,Work,Len);
	Add(Prod, Prod, Work, Len);

	Mul(Work, Num1 + HLen, Num2 + HLen, HLen);
	offset = Prod + HLen;RippleAdd(Prod,offset,offset,Work,Len);
	offset = Prod + Len; RippleAdd(Prod,offset,offset,Work,Len);
}


/*
*******************************
**     FFT Multiplication    **
*******************************
*/

#include "fft.c"

/*
** Do a convolution.
*/
void Convolution(double *FFTNum1, double *FFTNum2,int Len) {
	int x;
	FFTNum2[0] = FFTNum1[0] * FFTNum2[0];
	FFTNum2[1] = FFTNum1[1] * FFTNum2[1];
	for (x = 2; x < Len; x += 2) {
		double a, b, c, d;
		FFTNum2[x] = (a = FFTNum1[x]) * (b = FFTNum2[x]) -
			(c = FFTNum1[x + 1]) * (d = FFTNum2[x + 1]);
		FFTNum2[x + 1] = a * d + c * b;
	}
}

/*
** How we identify what the primary FFT cache holds.
*/
Short *CachedPtr=NULL;
int CachedLen=0;

/*
** Our secondary FFT cache stuff.
*/
Short *OldCachedPtr=NULL;
int OldCachedLen=0;
double *FFTNum3=NULL;

/*
** Flush our FFT Cache.
*/
void FlushFFTCache(void) {
	OldCachedPtr=NULL;
	OldCachedLen=0;
	CachedPtr=NULL;
	CachedLen=0;
}

/*
** Save the data from our regular FFT cache to our secondary FFT cache.
*/
void SaveFFTCache(void) {
	/*int x;*/
	OldCachedPtr=NULL;OldCachedLen=0;
	if (FFTNum1==NULL) return;
	if (FFTNum3==NULL) return;

	#if defined(USE_EXTRA_FFT_CACHE) && (FFTLimit!=0)
		OldCachedPtr=CachedPtr;
		OldCachedLen=CachedLen;
		memcpy(FFTNum3,FFTNum1,CachedLen*sizeof(double)*2);
	#endif
}

/*
** Swap the data in the primary FFT cache and the secondary FFT cache.
*/
void SwapFFTCache(void) {
	int x;Short *p;
	if ((OldCachedPtr==NULL) || (FFTNum1==NULL) || (FFTNum3==NULL)) {
		CachedPtr=NULL;
		CachedLen=0;
		return;
	}

	#if defined(USE_EXTRA_FFT_CACHE) && (FFTLimit!=0)
		x=CachedLen;
		p=CachedPtr;
		memcpy(FFTNum2,FFTNum1,CachedLen*sizeof(double)*2);
		CachedPtr=OldCachedPtr;
		CachedLen=OldCachedLen;
		memcpy(FFTNum1,FFTNum3,CachedLen*sizeof(double)*2);
		memcpy(FFTNum3,FFTNum2,x*sizeof(double)*2);
		OldCachedPtr=p;
		OldCachedLen=x;
		FFTCache=1;
	#endif
}

/*
** Restore the data from our secondary FFT cache to our primary FFT cache
*/
void RestoreFFTCache(void) {
	/*int x;
	double s;*/
	FFTCache=0;
	CachedPtr=NULL;CachedLen=0;
	if (OldCachedPtr==NULL) return;
	if (FFTNum1==NULL) return;
	if (FFTNum3==NULL) return;

	#if defined(USE_EXTRA_FFT_CACHE) && (FFTLimit!=0)
		CachedPtr=OldCachedPtr;
		CachedLen=OldCachedLen;
		memcpy(FFTNum1,FFTNum3,CachedLen*sizeof(double)*2);
		FFTCache=1;
	#endif
}

/*
** Do a Fast Fourier Transform based multiplication.
*/
void FFTMul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	int x, Len2 = Len * 2;
	double Carry, RawPyramid, Pyramid, PyramidError;
	double inv;

	/*
	** This is a radix-2 FFT, so the length has to be a power of two.
	*/
	
	/*if (!IsPow2(Len))
		FatalError("The FFT size is not a power of two\n");*/

	/*
	** Make sure we aren't trying to do too big of a FFT.
	** Note that this is 'element' size, not digits, like FFTLimit.
	*/
	if ((long) Len > 2097152L)
		FatalError("Too large of a FFT.  I fail over 8,388,608.\n");

	if (Len < 16) {
		SlowMul(Prod,Num1,Num2,Len);
		return;
	}

	/*
	** Safety check for the cache.
	*/
	if (FFTCache) {
		if ((Num1 != CachedPtr) || (Len  != CachedLen))
			FFTCache=0;
	}

	/*
	** If we are squaring a number, we can save the cost
	** of a FFT.
	*/
	if (Num1 == Num2) {
		if (!FFTCache) {
			LoadFFTNum(Num2,FFTNum2,Len);
			FwdRealFFT(FFTNum2, Len2);
		}
		if (FFTCmd) {
			memcpy(FFTNum1,FFTNum2,sizeof(double)*Len2);
			CachedPtr=Num2;CachedLen=Len;
		}
		Convolution(FFTNum2,FFTNum2,Len2);
	}
	else {/* Not squaring a number */
		if (!FFTCache && !FFTCmd) {
			LoadFFTNum(Num2,FFTNum2,Len);FwdRealFFT(FFTNum2, Len2);
			LoadFFTNum(Num1,FFTNum1,Len);FwdRealFFT(FFTNum1, Len2);
			Convolution(FFTNum1,FFTNum2,Len2);
			CachedPtr=Num1;CachedLen=Len;
		}
		else if (!FFTCache && FFTCmd) {
			LoadFFTNum(Num2,FFTNum1,Len);FwdRealFFT(FFTNum1, Len2);
			CachedPtr=Num2;CachedLen=Len;
			LoadFFTNum(Num1,FFTNum2,Len);FwdRealFFT(FFTNum2, Len2);
			Convolution(FFTNum1,FFTNum2,Len2);
		}
		else if (FFTCache && !FFTCmd) {
			LoadFFTNum(Num2,FFTNum2,Len);FwdRealFFT(FFTNum2, Len2);
			Convolution(FFTNum1,FFTNum2,Len2);
			CachedPtr=Num1;CachedLen=Len;
		}
		else if (FFTCache && FFTCmd) {
			memcpy(FFTNum2,FFTNum1,sizeof(double)*Len2);
			LoadFFTNum(Num2,FFTNum1,Len);FwdRealFFT(FFTNum1, Len2);
			CachedPtr=Num2;CachedLen=Len;
			Convolution(FFTNum1,FFTNum2,Len2);
		}
	}

	/*
	** Now do an Inverse FFT
	*/
	
	RevRealFFT(FFTNum2, Len2);

	/*
	** Now round the results, and release our carries, and store
	** the results in the 'prod' array.  Also do the normalization
	** we didn't do in the FFT.  And, as usual, it's slightly faster
	** to multiply by the reciprocal, instead of doing the division.
	*/
	Carry = 0.0;
	inv = 1.0 / Len;
		for (x = Len2; x > 0; x--) {
			RawPyramid = FFTNum2[Len2 - x] * inv + Carry;
			Pyramid = floor(RawPyramid + 0.5);
			PyramidError = fabs(RawPyramid - Pyramid);
			if (PyramidError > MaxFFTError) {
				MaxFFTError = PyramidError;
/*				printf("New Max FFT Error=%f\n",MaxFFTError);*/
        }
		Carry = floor(Pyramid / 10000);
		Prod[x - 1] = Pyramid - Carry * 10000;
	}

	/*
	** Do a bit of 'sanity' error checking.  This value
	** is based on some testing with a 'test jig' and
	** personal opinion.  Should be good enough to catch
	** systems with poor FPUs.  However, if the value ever
	** actually reaches 0.5, then you know for a FACT that
	** the FFTMul() has failed.
	**
	** The FFT fairly much depends upon at least 'long double'
	** (ie: 64 bits mantissa) in the butterfly and to hold
	** the trig vars.  If your compiler, for some reason,
	** doesn't keep those vars in the FPU registers, you
	** may need to uncomment the 'long' in some of the
	** 'double' expressions in the FFT.
	*/
	if (MaxFFTError >= 0.2) {
		printf("\n**WARNING** Max FFT Error: %f\n", MaxFFTError);
		puts("The FFT is approaching its limit.  As long as");
		puts("the FFTLimit is below 8,388,608 the results are");
		puts("almost certainly correct, but you may want to verify");
		puts("the results with an independant value.  If the");
		puts("value is very near 0.5, then the FFTMul() has");
		puts("definetly failed and the results are incorrect.");
		puts("If the value is just 0.4 or so, then the results");
		puts("are definetly questionable and it's up to you");
		puts("whether to trust them.  You could reduce the");
		puts("size of FFTLimit and try again.  This can also");
		puts("be caused by poor trig precision in the FFT.");
		puts("See program and docs for more details.");
		EXIT
    }

	FFTCache=0;
	FFTCmd=0;
}



/*
** FINALLY we get finished with the three specialized multiply
** routines and get to the generic one we actually call!
*/

/*
** This routine is the 'master' multiplication routine.  It decides
** which multiplication routine is most appropriate for the size
** of the numbers.  The regular slow O(N^2) method, the fractal
** O(N^1.585) method, or the FFT O(N Log2(N)) method.
*/
void Mul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	if (Len <= 16) {
		FFTCache=0;FFTCmd=0;
		SlowMul(Prod, Num1, Num2, Len);
		return;
	}
	
	/*
	** Do the FFT based multiplication, if the size is within
	** what we can handle.
	** Note: We have to convert our FFTLimit of digits to the
	** number of array elements used.  Simple division by 4.
	*/
	else if (Len <= FFTLimit / 4)
		FFTMul(Prod, Num1, Num2, Len);
	else {
		FFTCache=0;FFTCmd=0;
		FractalMul(Prod, Num1, Num2, Len);
	}
	FFTCache=0;FFTCmd=0;
}

/*
** Plain old fashioned multiplication.
*/
void DumbMul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	/*int x;*/

	Mul(MulProd,Num1,Num2,Len);
	CopyProd(Prod,MulProd,Len);
	FFTCache=0;FFTCmd=0;
}


int N1L0,N1R0,N2L0,N2R0;
/*
** Take care of simple, obvious multiplications.
*/
int EasyMul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	int x,Half;

	Half = Len/2;
	N1L0=CountZeros(Num1,     Half);
	N1R0=CountZeros(Num1+Half,Half);
	N2L0=CountZeros(Num2,     Half);
	N2R0=CountZeros(Num2+Half,Half);

	/* Do the quickie exact ones first. */
	if ( (N1R0+N1L0==Len) || (N2R0+N2L0==Len)) {
		for (x=0;x<Len;x++) Prod[x]=0;
	}
	else if ((Num1==Num2) && (N1L0==Half)) {
		for (x=0;x<Len;x++) Prod[x]=0;
    }
	else if ((Num1[0]==1000) && (N1R0+N1L0==Len-1)) {
		Copy(Prod,Num2,Len);
	}
	else if ((Num2[0]==1000) && (N2R0+N2L0==Len-1)) {
		Copy(Prod,Num1,Len);
    }
	else if ((Num1[0]==2000) && (N1R0+N1L0==Len-1)) {
		Add(Prod,Num2,Num2,Len);
	}
	else if ((Num2[0]==2000) && (N2R0+N2L0==Len-1)) {
		Add(Prod,Num1,Num1,Len);
	}
	else if ((Num1[0]==500) && (N1R0+N1L0==Len-1)) {
		Copy(Prod,Num2,Len);
		DivBy2(Prod,Len);
	}
	else if ((Num2[0]==500) && (N2R0+N2L0==Len-1)) {
		Copy(Prod,Num1,Len);
		DivBy2(Prod,Len);
	}
	else if (N2R0+N2L0 > Len-8) {
		SlowMul(MulProd,Num1,Num2,Len);
		CopyProd(Prod,MulProd,Len);
	}
	else if (N1R0+N1L0 > Len-8) {
		SlowMul(MulProd,Num2,Num1,Len);
		CopyProd(Prod,MulProd,Len);
	}
	else if ((N1R0==Half) && (N2R0==Half)) {
		Mul(MulProd,Num1,Num2,Half);
		CopyProd(Prod,MulProd,Len);
    }
	else return 0;

	FFTCache=0;FFTCmd=0;
	return 1;
}

/*
** Multiply two half precision numbers together and get a full
** precision result.  Just an optimization for the division and
** root routines.  Part of "Karp's trick".
**
** We could make this routine 'smart', but it isn't worth the effort.
** We'd only save, at most, part of one multiplication in the whole
** program run.
*/
void HalfMul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	/*int x;*/

	if (EasyMul(Prod,Num1,Num2,Len)) return;

	Mul(MulProd,Num1,Num2,Len/2);
	CopyProd(Prod,MulProd,Len);
	FFTCache=0;FFTCmd=0;
}


/*
** Basically, it's just a smart multiplication routine.
**
** It's just like you did:
**
** Mul(MulProd,Num1,Num2,Len);CopyProd(Dest,MulProd,Len);
**
** except you call it with the destination instead of 'MulProd' and
** you don't need to do the CopyProd() afterwards.  It is safe for
** the destination to be one of the inputs because it will not be
** written to until the multiplication is totally finished.
**
** Plus, this routine will attempt to recognise that certain numbers
** can be converted into faster forms for multiplication.  For example,
** if half the number is zero, we can take advantage of it.  If it's
** 0.5 or 1.0 etc., we can recognise it and just do it directly.  And
** so on.
**
** The '-8' means there can be 8 non zero/nine digits.  Arbitrary,
** but of course we don't want it to be too high.  Examination of
** the numbers show it's a good size.
**
** Any call to SmartMul could be replaced by the regular Mul() and
** CopyProd() calls I show above.  Or you could call DumbMul() with
** the same params as SmartMul(), just that it won't do any tricks.
** (just used for debugging, testing, etc.  If DumbMul vs. SmartMul
** ever return different results, then something is wrong.)
*/
void SmartMul(Short *Prod, Short *Num1, Short *Num2, int Len) {
	int Half=Len/2;
	int x;

	if (Len <= 16) {
		Mul(MulProd,Num1,Num2,Len);
		CopyProd(Prod,MulProd,Len);
		return;
	}

	if (EasyMul(Prod,Num1,Num2,Len)) { }
	else if ((N2L0 > Half-8) && (N1R0 > Half-8)) {
		for (x=0;x<Len*2;x++) MulProd[x]=0;
		Mul(MulProd+Half,Num1,     Num2+Half,Half);
		AddSlowMul(MulProd,     Num1,     Num2,     Half);
		AddSlowMul(MulProd+Half,Num1+Half,Num2,     Half);
		AddSlowMul(MulProd+Len, Num2+Half,Num1+Half,Half);
		CopyProd(Prod,MulProd,Len);
	}
	else if ((N2L0 > Half-8) && (Num1!=Num2)) {
		int Carry;
		FFTCmd=1;
		Mul(MulProd, Num1, Num2+Half,Half);
		FFTCache=1;
		Mul(MulProd+Len, Num2+Half,Num1+Half,Half);
		Carry=RippleAdd(MulProd,MulProd+Half,MulProd+Half,MulProd+Len,Half);
		Copy(MulProd+Half,MulProd,Len);
		for (x=0;x<Half;x++) MulProd[x]=0;
		MulProd[Half-1]=Carry;
		AddSlowMul(MulProd+Half,Num1+Half,Num2,     Half);
		AddSlowMul(MulProd,     Num1,     Num2,     Half);
		CopyProd(Prod,MulProd,Len);
	}
	else if (N1L0 > Half-8) {
		int Carry;
		for (x=0;x<Len*2;x++) MulProd[x]=0;
		FFTCmd=0;
		Mul(MulProd+Len, Num1+Half,Num2+Half,Half);
		FFTCache=1;
		Mul(MulProd,     Num1+Half,Num2,     Half);
		Carry=RippleAdd(MulProd,MulProd+Half,MulProd+Half,MulProd+Len,Half);
		Copy(MulProd+Half,MulProd,Len);
		for (x=0;x<Half;x++) MulProd[x]=0;
		MulProd[Half-1]=Carry;
		AddSlowMul(MulProd+Half,Num2+Half,Num1,     Half);
		AddSlowMul(MulProd,     Num2,     Num1,     Half);
		CopyProd(Prod,MulProd,Len);
	}
	else if (N1R0 > Half-8) {
		for (x=0;x<Len*2;x++) MulProd[x]=0;
		Mul(MulProd, Num1, Num2, Half);
		FFTCache=1;
		Mul(MulProd+Len, Num1, Num2+Half,Half);
		RippleAdd(MulProd,MulProd+Half,MulProd+Half,MulProd+Len,Half);
		Copy(MulProd+Len,MulProd+Half+Len,Half);
		for (x=Len+Half;x<Len*2;x++) MulProd[x]=0;
		AddSlowMul(MulProd+Len, Num2+Half,Num1+Half,Half);
		AddSlowMul(MulProd+Half,Num2, Num1+Half,Half);
		CopyProd(Prod,MulProd,Len);
	}
	else DumbMul(Prod,Num1,Num2,Len);

	FFTCache=0;FFTCmd=0;
}

/*
************************************
**  Division and root extraction  **
************************************
*/

/*
** d = a/b by computing the reciprocal of b and then multiplying
** that by a.
*/
void Divide(Short *R, Short *Num1, Short *Num2, int Len, int SubLen) {
	int x, Sign;
	int Redo=REDO_LEN;

	SubLen = SubLen & -SubLen; /* make it a pow of two if it isn't already */
	if (SubLen <= 0) SubLen = 4;
	if (SubLen > Len) SubLen = Len;

	for (x = 0; x < Len; x++) R[x] = 0;
	/* There are only a few divisors, so I can cheat. */
	if ((Num2[0]== 318) && (Num2[1]==3098) && (Num2[2]==8618) && (Num2[3]==3790)) {
		R[0]=3141;
		R[1]=5926;
		R[2]=5358;
		R[3]=9793;
	}
	else if ((Num2[0]== 913) && (Num2[1]==8931) && (Num2[2]==6208) && (Num2[3]==8927)) {
		R[0]=1094;
		R[1]=2198;
		R[2]= 761;
		R[3]=3238;
	}
	else if ((Num2[0]==2007) && (Num2[1]==4977) && (Num2[2]==7425) && (Num2[3]==4721)) {
		R[0]= 498;
		R[1]=1325;
		R[2]=5726;
		R[3]=8337;
	}
	else if ((Num2[0]==2000) && (Num2[1]==   0) && (Num2[2]==   4) && (Num2[3]==8646)) {
		R[0]= 499;
		R[1]=9999;
		R[2]=9998;
		R[3]=7838;
	}
	else if ((Num2[0]==2000) && (Num2[1]==   0) && (Num2[2]==   0) && (Num2[3]==   0)) {
		R[0]= 500;
		R[1]=   0;
		R[2]=   0;
		R[3]=   0;
		}
	else {
		PrintShort("Unknown Divisor: ",Num2,16);
		EXIT;
	}

	if (SubLen >= Redo) Redo=0;
	FFTCache = 0;
	FlushFFTCache();
	while (SubLen < Len/2) {
		/*fputc('R', stderr);*/
		SubLen *= 2;if (SubLen > Len) SubLen = Len;
		FlushFFTCache();

		SmartMul(DSWork, R, Num2, SubLen);
		if (SubLen==Len/2) SaveFFTCache();

		MulProd[0] = 1000; for (x = 1; x < SubLen; x++) MulProd[x] = 0;
		Sign=Sub(DSWork, MulProd, DSWork, SubLen);
		if (Sign) Negate(DSWork,SubLen);

		FFTCache = 1;
		SmartMul(DSWork,R,DSWork,SubLen);
		if (Sign) Sub(R,R,DSWork,SubLen);
		else Add(R,R,DSWork,SubLen);

		if (Redo == -1) Redo=0;
		if (SubLen == Redo) {
			SubLen/=2;
			Redo=-1;
		}
	}

	for (x=Len/2;x<Len;x++) R[x]=0;
	HalfMul(OldRoot,R,Num1,Len);
	SaveFFTCache();

	for (x=Len/2;x<Len;x++) OldRoot[x]=0;
	SmartMul(DSWork,OldRoot,Num2,Len);

	Sign=Sub(DSWork,Num1,DSWork,Len);
	if (Sign) Negate(DSWork,Len);

	RestoreFFTCache();
	SmartMul(DSWork,R,DSWork,Len);
	if (Sign) Sub(R,OldRoot,DSWork,Len);
	else Add(R,OldRoot,DSWork,Len);
	FFTCache = 0;
}

/*
** Computes the square root of 'n' by computing its reciprocal square
** root using the Newton's method, and then multiplying by the original
** number.  This is faster than doing the normal Newton 'divide & average'
** method.
*/
void Sqrt(Short *Root, Short * Num, int Len, int SubLen) {
	int x,Sign;
	int Redo=REDO_LEN;

	SubLen = SubLen & -SubLen; /* make it a pow of two if it isn't already */
	if (SubLen <= 0) SubLen = 4;
	if (SubLen > Len) SubLen = Len;

	if ((Num[0]== 500) && (Num[1]==0000) && (Num[2]==0000) && (Num[3]==0000)) {
		OldRoot[0]=1414;
		OldRoot[1]=2135;
		OldRoot[2]=6237;
		OldRoot[3]=3095;
	}
	else if ((Num[0]== 707) && (Num[1]==1067) && (Num[2]==8118) && (Num[3]==6547)) {
  		OldRoot[0]=1189;
  		OldRoot[1]=2071;
  		OldRoot[2]=1500;
  		OldRoot[3]=2721;
  	}
	else if ((Num[0]== 717) && (Num[1]==7499) && (Num[2]==8637) && (Num[3]==7537)) {
		OldRoot[0]=1180;
		OldRoot[1]=3570;
		OldRoot[2]=6419;
		OldRoot[3]=5417;
	}
	else if ((Num[0]== 717) && (Num[1]==7700) && (Num[2]==1097) && (Num[3]==6296)) {
		OldRoot[0]=1180;
		OldRoot[1]=3405;
		OldRoot[2]=9907;
		OldRoot[3]=3515;
	}
	else if ((Num[0]== 717) && (Num[1]==7700) && (Num[2]==1104) && (Num[3]==6129)) {
		OldRoot[0]=1180;
		OldRoot[1]=3405;
		OldRoot[2]=9901;
		OldRoot[3]=6096;
	}
	/* For the Borwein sqrt(2.0). */
	else if ((Num[0]==2000) && (Num[1]==   0) && (Num[2]==   0) && (Num[3]==   0)) {
		OldRoot[0]= 707;
		OldRoot[1]=1067;
		OldRoot[2]=8118;
		OldRoot[3]=6547;
	}
	else {
		PrintShort("Unknown Square root: ",Num,16);
		EXIT
	}
	for (x=SubLen;x<Len;x++) OldRoot[x]=0;

	if (SubLen >= Redo) Redo=0;
	FFTCache=0;
	FlushFFTCache();
	while (SubLen < Len/2) {
		/*fputc('s', stderr);*/
		SubLen *= 2;if (SubLen > Len) SubLen = Len;
		FlushFFTCache();

		for (x=SubLen/2;x<SubLen;x++) OldRoot[x]=0;
		FFTCmd=1;
		SmartMul(DSWork, OldRoot, OldRoot, SubLen);
		SaveFFTCache();
		SmartMul(DSWork, Num, DSWork, SubLen);

		MulProd[0]=1000;
		for (x=1;x<SubLen;x++) MulProd[x]=0;
		Sign=Sub(DSWork,MulProd,DSWork,SubLen);
		if (Sign) Negate(DSWork,SubLen);

		if (SubLen==Len/2) SwapFFTCache();	/* Restore & save 'Num' */
		else RestoreFFTCache();				/* Restore */
		SmartMul(DSWork,OldRoot,DSWork,SubLen);
		DivBy2(DSWork,SubLen);

		if (Sign) Sub(OldRoot,OldRoot,DSWork,SubLen);
		else Add(OldRoot,OldRoot,DSWork,SubLen);

		if (Redo == -1) Redo = 0;
		if (SubLen == Redo) {SubLen/=2;Redo=-1;}
	}
	/*fputc('s', stderr);*/

	RestoreFFTCache(); /* Restore 'Num' */
	FFTCmd=1;
	HalfMul(Root, Num, OldRoot, Len);
	for (x=Len/2;x<Len;x++) Root[x]=0;
	HalfMul(DSWork, Root, Root, Len);

	Sign=Sub(DSWork, Num, DSWork, Len);
	if (Sign) Negate(DSWork,Len);

	FFTCache = 1;
	SmartMul(DSWork,OldRoot,DSWork,Len);

	DivBy2(DSWork, Len);
	if (Sign) Sub(Root, Root, DSWork, Len);
	else Add(Root, Root, DSWork, Len);

	FFTCache = 0;
}

/*
** Calculate the 4th root of a number.  Same basic method as Sqrt()
** except it's easier to leave it in reciprocal form.
*/
void FourthRoot(Short *Root, Short * Num, int Len, int SubLen) {
	int x,Sign;
	int Redo=REDO_LEN;

	SubLen = SubLen & -SubLen;	/* make it a pow of two if it isn't already */
	if (SubLen <= 0) SubLen = 4;
	if (SubLen > Len) SubLen = Len;

	for (x=0;x<Len;x++) Root[x]=0;
	if ((Num[0]== 970) && (Num[1]==5627) && (Num[2]==4847) && (Num[3]==7140)) {
		Root[0]=1007;
		Root[1]=4977;
		Root[2]=7425;
		Root[3]=4721;
	}
	else if ((Num[0]== 999) && (Num[1]==9999) && (Num[2]==9980) && (Num[3]==5415)) {
		Root[0]=1000;
		Root[1]=   0;
		Root[2]=   4;
		Root[3]=8646;
	}
	else if ((Num[0]== 999) && (Num[1]==9999) && (Num[2]==9999) && (Num[3]==9999)) {
		Root[0]=1000;
	}
	else {
		PrintShort("Unknown Fourth Root: ",Num,16);
		EXIT;
	}

	if (SubLen >= Redo) Redo=0;
	FFTCache=0;
	FlushFFTCache();
	while (SubLen < Len) {
		/*fputc('s', stderr);*/
		SubLen *= 2;if (SubLen > Len) SubLen = Len;
		FlushFFTCache();

		for (x=SubLen/2;x<SubLen;x++) Root[x]=0;
		FFTCmd=1;
		SmartMul(DSWork, Root, Root, SubLen);
		SaveFFTCache();
		SmartMul(DSWork, DSWork, DSWork, SubLen);

		SmartMul(DSWork, Num, DSWork, SubLen);

		OldRoot[0]=1000;
		for (x=1;x<SubLen;x++) OldRoot[x]=0;
		Sign=Sub(DSWork,OldRoot,DSWork,SubLen);
		if (Sign) Negate(DSWork,SubLen);

		RestoreFFTCache();
		SmartMul(DSWork,Root,DSWork,SubLen);
		DivBy2(DSWork, SubLen);DivBy2(DSWork, SubLen);

		if (Sign) Sub(Root,Root,DSWork,SubLen);
		else Add(Root,Root,DSWork,SubLen);

		if (Redo == -1) Redo = 0;
		if (SubLen == Redo) {SubLen/=2;Redo=-1;}
	}

	FFTCache = 0;
}

/*
** The Borwein-Borwein-Bailey quartic (4) pi program.
**
** Root() means the 4th root.  ie: sqrt(sqrt(x))
**
** Set:
**
** a=6-4*sqrt(2)
** b=sqrt(2)-1
**
** Iterate:
**
**    1-Root(1-b^4)
** b= -------------
**    1+Root(1-b^4)
**
** a=(1+b)^4*a-2^(2n+3)*b*(1+b+b^2)
**
** and 'a' converges to 1/pi
**
** 'b' must be computed before 'a'.
**
** All of those powers can be done fairly efficiently by raising
** b to 2, 3, and the 4th power.  Even the (1+b)^4 decomposes into
** powers of b, so we don't need to do a special 4th power.
**
** a=(1+b)^4*a-2^(2n+3)*b*(1+b+b*b)
**
** (1+b)^4     = (1+4b+6b^2+4b^3+b^4)
** b*(1+b+b^2) = (b+b^2+b^3)
**
** b^2 = b*b     FU   (b   -> cache)
** b^3 = b*b^2   CFU  (b^2 -> cache)
** b^4 = b^2*b^2 CCU
**
*/
int Borwein(int Len, int MaxPasses) {
	int x;
	double Pow2/*,carryd*/;
	Short *VAR_A,*VAR_B;
	Short *Work1,*Work2,*Work3;
	int Pass;
	int Done;
	
	sprintf(formula, "using Borwein's Quartic Formula.");

	VAR_A = (Short *) calloc(Len, sizeof(Short));
	VAR_B = (Short *) calloc(Len, sizeof(Short));
	Work1 = (Short *) calloc(Len, sizeof(Short));
	Work2 = (Short *) calloc(Len, sizeof(Short));
	Work3 = (Short *) calloc(Len, sizeof(Short));
	
	if (Work3 == NULL) printf("Work3\n");
	
	if ((VAR_A == NULL) || (VAR_B == NULL) ||
    	(Work1 == NULL) || (Work2 == NULL) || (Work3 == NULL))
		FatalError("Unable to allocate enough memory for the Borwein work variables.\n");

	FFTCache = 0; FFTCmd = 0;
	Pow2=8;
	Pass=0;
	StartTime = clock();

	/* a=6-4*sqrt(2) b=sqrt(2)-1 */
	fprintf(stderr,"Initialization:\n");
	LoopTime = clock();
	Work3[0]=2000;for (x=1;x<Len;x++) Work3[x]=0;
	Sqrt(Work1,Work3,Len,0);
	VAR_A[0]=6000;for (x=1;x<Len;x++) VAR_A[x]=0;
	Sub(VAR_A,VAR_A,Work1,Len);Sub(VAR_A,VAR_A,Work1,Len);
	Sub(VAR_A,VAR_A,Work1,Len);Sub(VAR_A,VAR_A,Work1,Len);
	/* 17-12*sqrt(2) is same as (sqrt(2)-1)^4, but a lot faster */
	Work3[0]=9000;for (x=1;x<Len;x++) Work3[x]=0;
	MulBy(Work1,6,Len);
	Sub(Work3,Work3,Work1,Len);
	Work3[0]+=8000; /* We only have one digit (0-9) of integer... */
	Sub(Work3,Work3,Work1,Len);
	NextLoopTime = clock();
	fprintf(stderr, " Time= %6.2f Seconds\n", (float)(NextLoopTime-LoopTime)/(float)CLOCKS_PER_SEC);

	MaxPasses+=Pass;
	Done=0;
	while ((!Done) && (Pass < MaxPasses)) {
		if (IsZero(Work3,Len)) Done=1;
		if (Done) break;

		fprintf(stderr,"Pass %d\n",++Pass);
		LoopTime = clock();

		/*
		** b= (1-Root(1-b^4)) / (1+Root(1-b^4)) Work3 is already b^4
		** Slightly different because FourthRoot returns reciprocal, not root.
		** b= (Root(1-b^4)-1) / (Root(1-b^4)+1)
		*/
		Work2[0]=1000;for (x=1;x<Len;x++) Work2[x]=0;
		Sub(Work1,Work2,Work3,Len);
		FourthRoot(Work2,Work1,Len, (Pass>2) ? (1<<(Pass*2-3)) : 0 );
		Copy(Work1,Work2,Len);
		Work1[0]-=1000;
		Work2[0]+=1000;
		Divide(VAR_B,Work1,Work2,Len, (Pass>2) ? (1<<(Pass*2-3)) : 0 );

		/* Work1=1+4*b */
		FlushFFTCache();
		Work1[0]=1000;for (x=1;x<Len;x++) Work1[x]=0;
		Add(Work1,Work1,VAR_B,Len);Add(Work1,Work1,VAR_B,Len);
		Add(Work1,Work1,VAR_B,Len);Add(Work1,Work1,VAR_B,Len);
		/* Work2=b */
		Copy(Work2,VAR_B,Len);

		/* z=b^2 */
		FFTCmd = 1;
		SmartMul(Work3,VAR_B,VAR_B,Len);
		/* Work1=Work1+6*b^2 */
		Add(Work1,Work1,Work3,Len);Add(Work1,Work1,Work3,Len);
		Add(Work1,Work1,Work3,Len);Add(Work1,Work1,Work3,Len);
		Add(Work1,Work1,Work3,Len);Add(Work1,Work1,Work3,Len);
		/* Work2=Work2+b^2 */
		Add(Work2,Work2,Work3,Len);

		/* z=b^3 */
		FFTCache = 1;
		FFTCmd = 1;
		SmartMul(VAR_B,VAR_B,Work3,Len);
		/* Work1=Work1+4*b^3 */
		Add(Work1,Work1,VAR_B,Len);Add(Work1,Work1,VAR_B,Len);
		Add(Work1,Work1,VAR_B,Len);Add(Work1,Work1,VAR_B,Len);
		/* Work2=Work2+b^3 */
		Add(Work2,Work2,VAR_B,Len);

		/* b=b^4 */
		FFTCache = 1;
		/* The cache ends up in FFTNum1, but I need it in FFTNum2 for the squaring.*/
		if (FFTNum1) memcpy(FFTNum2,FFTNum1,sizeof(double)*Len*2);
		SmartMul(Work3,Work3,Work3,Len);
		/* Work1=Work1+b^4 */
		Add(Work1,Work1,Work3,Len);

		/* Work1=Work1*a */
		SmartMul(Work1,Work1,VAR_A,Len);
		/* Work2=Work2*2^(2n+3) */
		MulByFloat(Work2,Pow2,Len);Pow2*=4;

		/* a=Work1-Work2 */
		Sub(VAR_A,Work1,Work2,Len);
		NextLoopTime = clock();
		fprintf(stderr, "Time= %6.2f Seconds\n", (float)(NextLoopTime-LoopTime)/(float)CLOCKS_PER_SEC);
	}

	Work3[0]=1000;
	for (x=1;x<Len;x++) Work3[x]=0;
	Divide(Work1,Work3,VAR_A,Len,0);
	EndTime = clock();

	printf("\a\nTotal Execution time: %6.2f Seconds\n\n",
		(float)(EndTime - StartTime)/(float)CLOCKS_PER_SEC);
	
	do {
		printf("Would you like to dump Pi to screen (y or n)?\n");
		gets(noy);
	} while((tolower(noy[0]) != 'n') && (tolower(noy[0]) != 'y'));
	if(tolower(noy[0]) == 'y') PrintShort("", Work1, Len);

  	do {
		printf("\nWould you like to save Pi to disk (y or n)?\n");
		gets(yon);
	} while((tolower(yon[0]) != 'n') && (tolower(yon[0]) != 'y'));
	if(tolower(yon[0]) == 'y') SaveShort("", Work1, Len);


	free(VAR_A);
	free(VAR_B);
	free(Work1);
	free(Work2);
	free(Work3);

	return Done;
}


void SaveShort(char *Str, Short *Num, int Len) {
	FILE *disk;
	int x;
	int printed = 0;
	MakeName();
	disk = fopen(filename, "w");
	if (disk == 0) {
		printf("An error occurred while writing Pi to disk.\n\a");
		return;
	}
	fprintf(disk, "%d Digits of Pi calculated %s\n\n", name, formula);
	
	fprintf(disk, "%s", Str);

	fprintf(disk, "%u.\n\n", Num[0]/1000);
	SaveDig(disk, (Num[0]/100)%10,++printed);
	SaveDig(disk, (Num[0]/10)%10,++printed);
	SaveDig(disk, Num[0]%10,++printed);

	for (x = 1; x < Len; x++) {
		if (Num[x]<0)      fprintf(disk, "-");
		if (Num[x]>=10000) fprintf(disk, "+");
		SaveDig(disk, (Num[x]/1000)%10,++printed);
		SaveDig(disk, (Num[x]/100)%10,++printed);
		SaveDig(disk, (Num[x]/10)%10,++printed);
		SaveDig(disk, Num[x]%10,++printed);
	}
	fprintf(disk, "\n");
	
	fprintf(disk, "\n\nTotal Execution time: %6.2f Seconds\n\n",
		(float)(EndTime-StartTime)/(float)CLOCKS_PER_SEC);
	fclose(disk);
	/*if(fclose(disk) == 0) {
		printf("An error occurred while saving Pi to disk.\n\a");
	}
	else printf("Pi has been saved to disk.\n");*/	
}

void SaveDig(FILE *disk, int Dig, long printed) {
	/* Local function for PrintShort() */
	fprintf(disk, "%d",Dig);
	if ((printed % 1000) == 0)
	    fprintf(disk, "\n\n\n\n");
	else if ((printed % 50) == 0)
	    fprintf(disk, "\n");
	else if ((printed % 10) == 0)
	    fprintf(disk, " ");
}

/*
*****************************************************
**      The AGM itself                             **
**                                                 **
** A[n] = (A[n-1] + B[n-1])/2                      **
** B[n] = Sqrt(A[n-1]*B[n-1])                      **
** C[n] = (A[n-1]-B[n-1])/2                        **
**                                                 **
**                          n                      **
** PI[n] = 4A[n+1]^2 / (1-(Sum (2^(j+1))*C[j]^2))  **
**                        j = 1                    **
*****************************************************
*/
int AGM(int Len, int MaxPasses) {
	int /*x,*/ Sign, Pass;
	double Pow2;
	Short *AGM_A=NULL, *AGM_B=NULL, *AGM_C=NULL, *AGMWork=NULL;
	int Done; /* boolean */
	/*int SkipLen;*/
	
	sprintf(formula, "using Gauss's Arithmetic Geometric Formula.");

	AGM_A = (Short *) calloc(Len, sizeof(Short));
	AGM_B = (Short *) calloc(Len, sizeof(Short));
	AGM_C = (Short *) calloc(Len, sizeof(Short));
	AGMWork = (Short *) calloc(Len, sizeof(Short));

	if ((AGM_A == NULL) || (AGM_B == NULL) ||
		(AGM_C == NULL) || (AGMWork == NULL))
		FatalError("Unable to allocate enough memory for the AGM work variables.\n");

	FFTCache = 0; FFTCmd = 0;
	Pow2 = 4.0;
	Pass = 0;
	
	StartTime = clock();

	fprintf(stderr, "Initialization:\n");
	LoopTime = clock();
	AGM_A[0] = 1000;
	AGM_C[0] = 1000;
	AGMWork[0]=500;
	Sqrt(AGM_B,AGMWork,Len,0);
	NextLoopTime = clock();
	fprintf(stderr, " Time= %6.2f Seconds\n", (float)(NextLoopTime-LoopTime)/(float)CLOCKS_PER_SEC);     

	Done=0;
	MaxPasses+=Pass;
	while ((!Done) && (Pass < MaxPasses)) {
      fprintf(stderr, "Pass %d\n", ++Pass);
      LoopTime = clock();

      /* w = (a-b)/2 */
      Sign = Sub(AGMWork, AGM_A, AGM_B, Len);
      if (Sign) Negate(AGMWork,Len);
      DivBy2(AGMWork, Len);

      /* m = w*w */
      SmartMul(AGMWork, AGMWork, AGMWork, Len);

      /* m = m* w^(J+1) */
      MulByFloat(AGMWork,Pow2,Len);
      Pow2 *= 2.0;

      /* c = c - m */
      if (Sign) Add(AGM_C, AGM_C, AGMWork, Len);
      else      Sub(AGM_C, AGM_C, AGMWork, Len);

      /* See if it's done */
      if (IsZero(AGMWork,Len-(Len/16))) Done=1;

      /* m = a*b */
      if (!Done)   /* If last iter, we can skip it. */
         SmartMul(AGMWork, AGM_A, AGM_B, Len);

      /* a = (a+b)/2 */
      Add(AGM_A, AGM_A, AGM_B, Len);
      DivBy2(AGM_A, Len);

      /* b = sqrt(a*b) */
      if (!Done) /* Optimization */
          Sqrt(AGM_B, AGMWork, Len, (Pass>5) ? (4<<(Pass-5)) : 0 );
		
		NextLoopTime = clock();
      fprintf(stderr, " Time= %6.2f Seconds\n", (float)(NextLoopTime-LoopTime)/(float)CLOCKS_PER_SEC);
    }
	EndTime = clock();

	SmartMul(AGM_A, AGM_A, AGM_A, Len);
	MulBy(AGM_A,4,Len);

	Divide(AGM_B, AGM_A, AGM_C, Len,0);

	printf("\a\nTotal Execution time: %6.2f Seconds\n\n", (float)(EndTime - StartTime)/(float)CLOCKS_PER_SEC);
	
	do {
		printf("Would you like to dump Pi to screen (y or n)?\n");
		gets(noy);
	} while((tolower(noy[0]) != 'n') && (tolower(noy[0]) != 'y'));
	if(tolower(noy[0]) == 'y') PrintShort("", AGM_B, Len);

  	do {
		printf("\nWould you like to save Pi to disk (y or n)?\n");
		gets(yon);
	} while((tolower(yon[0]) != 'n') && (tolower(yon[0]) != 'y'));
	if(tolower(yon[0]) == 'y') SaveShort("", AGM_B, Len);

	free(AGM_A);
	free(AGM_B);
	free(AGM_C);
	free(AGMWork);

	return (Done);
}

int main() {
	char thedigits[100], method[100], morepi[100], nyo[100];
	int Done;
	int Len, Digits, Pow;
	int MaxPasses;
	
	printf("\n");
	printf("                                                             \n");
	printf("  PPPPPP  IIIIIII            AAAAA    GGGGG  MMM         MMM \n");
	printf("  PPP   P   III             AAA   A  GGG   G MMMM       MMMM \n");
	printf("  PPP   P   III             AAA   A  GGG     MMM M     M MMM \n");
	printf("  PPPPPP    III             AAAAAAA  GGG     MMM  M   M  MMM \n");
	printf("  PPP       III             AAA   A  GGG  GG MMM   M M   MMM \n");
	printf("  PPP       III             AAA   A  GGG   G MMM    M    MMM \n");
	printf("  PPP     IIIIIII           AAA   A   GGGGG  MMM         MMM \n");
	printf("                                                             \n");
	printf("\n\n");
	printf("Pi AGM is a Pi Calculation program by Carey Bloodworth.\n\n");
	
	do {
	
		do {
			printf("How many digits of Pi do you want to calculate (must be a number)?\n");
			printf("It will be rounded up to the nearest power of 2 greater than 64.\n");
			Digits = atoi(gets(thedigits));
			for(Pow = 2; Pow < Digits; Pow *= 2);
			Digits = Pow;
			if(Digits < 64) Digits = 64;
			printf("Are you sure you want to calculate %d digits of Pi (y or n)?\n", Digits);
			gets(nyo);
		} while(tolower(nyo[0]) == 'n');
		
		Len = (Digits + 3) / 4;
			
		do {
			printf("Would you like to use the Borwein Quartic Formula\n");
			printf("or the Gauss Arithmetic Geometric Mean (b or g)?\n");
			gets(method);
		} while(tolower(method[0]) != 'b' && tolower(method[0]) != 'g');
	
		name = Digits;

		MaxPasses=Log2(Len)+4;
	
		DSWork = (Short *) calloc(Len, sizeof(Short));
		OldRoot = (Short *) calloc(Len, sizeof(Short));
		if (Digits <= FFTLimit) MulProd = (Short *) malloc(Len * 2* sizeof(Short));
		else MulProd = (Short *) malloc(Len * 4* sizeof(Short));

		if ((DSWork == NULL) || (OldRoot == NULL) || (MulProd == NULL))
			FatalError("Unable to allocate work memory for %d digits\n",Digits);

		FFTNum1 = FFTNum2 = FFTNum3=NULL;
		FFTArraySize = 0;
		#if FFTLimit !=0
			FFTArraySize = Min(FFTLimit / 2, Len * 2);
			FFTNum1 = (double *) malloc(FFTArraySize* sizeof(double));
			FFTNum2 = (double *) malloc(FFTArraySize* sizeof(double));
  			if ((FFTNum1 == NULL) || (FFTNum2 == NULL))
			FatalError("Unable to allocate FFT memory for %d digits\n",Digits);
			#ifdef USE_EXTRA_FFT_CACHE
			FFTNum3 = (double *) malloc(FFTArraySize* sizeof(double));
			if (FFTNum3 == NULL)
				FatalError("Unable to allocate FFT cache memory for %d digits\n",Digits);
			#endif
		#endif

		InitFFT(Len);

		/*
		** Now actually do the pi formula
		*/
		printf("Calculating %d Digits of Pi...\n", Digits);
		if (method[0] == 'b') Done=Borwein(Len,MaxPasses);
		else Done=AGM(Len,MaxPasses);

		/*
		printf("Max FFT Error: %f\n", MaxFFTError);
		*/
		

		free(DSWork);
		free(OldRoot);
		free(MulProd);
		free(FFTNum1);
		free(FFTNum2);
		#ifdef USE_EXTRA_FFT_CACHE
			free(FFTNum3);
		#endif
		
		do {
			printf("Would you like to calculate more Pi (y or n)?\n");
			gets(morepi);
		} while(tolower(morepi[0]) != 'y' && tolower(morepi[0]) != 'n');
	} while(morepi[0] == 'y');

	QUIT
	EXIT
	return 1;
}

/*
** The FFTLimit is the largest FFT based multiplication you can do
** in physical memory.  You do *NOT* want disk based virtual memory
** to kick in in the middle of a FFT, although virtual memory usage
** elsewhere is some what tolerable.
**
** This is a matter of obtaining the best performance possible, not
** of program ability.  There is no actual limit to the number of
** digits of pi this program can generate, regardless of the setting
** of FFTLimit.  If you had enough time, you could even compute
** 100 million digits on a 4 meg computer.
**
** You need to set the FFTLimit according to the following guide:
**
** With virtual memory (either OS or from the compiler), basically,
** you should divide the physical memory given to your program by
** 4, then round down to the next power of two, and that's the number
** of digits that can be multiplied together in a single 'chunk'.
** Numbers larger than that will first be broken into smaller chunks
** that the FFT can handle.
**
** For a 4meg (4194304) computer, you'd have 4194304/4=1,048,576
** But, even though that's a power of two, you have to go down to
** the next one, so you'll still have enough room for your OS and
** the program.  So it'd be 524,288
**
** Assuming virtual memory:
** A     4 meg computer can manage a threshold of   524,288.
** A  5- 8 meg computer can manage a threshold of 1,048,576.
** A  9-16 meg computer can manage a threshold of 2,097,152.
** A 17-32 meg computer can manage a threshold of 4,194,304.
** A 33-?? meg computer can manage a threshold of 8,388,608.
**
** If you don't have virtual memory, then you just have to determine
** yourself just how much you can spare to the FFT.  16 megs of
** memory is enough to do 1,048,576 digits of pi entirely within
** physical memory and with the FFTLimit set at 1,048,576.  You could
** just set it extremely high, and if you try to compute too many,
** it'll tell you when you try.  However, since the FFT uses most of
** the memory, reducing this to a level smaller than the number of
** digits you are computing will actually let you compute more digits.
** Reducing the FFTLimit to '0' would turn off the FFT (and its large
** memory usage) and let you compute even more digits, but increase
** the run time considerably.  Without the FFT, the program isn't even
** really practical.  Other programs, such as a classic arctangent,
** would be better.  I can't help you with this.  You'll have to decide
** for yourself.
**
** If you've got enough memory, it is definetly best to set it as
** high as possible.  This FFT based multiplication implementation has
** a *MAX* size of 8,388,608.  Beyond that, it fails due to the limited
** precision of an 8 byte 'double' (ie: it overflows.)  Changing all the
** double's to "long double" would probably fix that (and increase the
** limit to several billion digits), but the memory and execution cost
** would significantly increase, and C doesn't even provide for 'long
** double' functions.
**
** Examples:
#define FFTLimit   524288
#define FFTLimit  1048576
#define FFTLimit  2097152
#define FFTLimit  4194304
#define FFTLimit  8388608
**
**
** To compile this program, just give it to your compiler.  You may
** also need to link in the math libraries if your compiler doesn't
** automatically do this.  You might also want to turn on various
** optimizations.  For the DJGPP GNU C compiler, my command line was:
**
** gcc -O3 -m486 -ffast-math pi_agm.c -o pi_agm.exe -lm
**
** The -O3 turns on max optimizations, the -m486 turns on 486 code
** generation, the -ffast-math uses the FPU math sine & cosine instead
** of the C library, and the -lm links in the math libraries for the
** other functions.
**
**
** Total Memory usage:
** There are 3 actual formula variables of length   SIZE
** There are 3 temporary / work variables of length SIZE
** There is 1 multiplication variable of length     SIZE*4
**   (Can be just SIZE*2 if it fits entirely within the FFT.)
** There are 2 FFTNum's for the FFT                 FFTLimit/2
**
** SIZE is (DIGITS_WANTED+3)/4.
** Each array element is a short 16 bit integer.
**
** So, to compute 1024k digits totally within memory and the numbers
** fitting entirely within the FFT, SIZE would be 256k, you'd have 6
** 'SIZE' variables, plus a 2*SIZE multiply product array.  That's
** a total of 8*256k*2 (2 bytes per short)=4096k.  The FFTLimit would
** of course be 1024k.  Each FFTNum is actually half that many elements
** (four digits per element, but it has to be twice as long, so it can
** hold the product), and we have two FFTNum's, so that'd be 1meg*8=8meg,
** plus the 4meg we need for the algorithm itself, for a total of 12meg.
**
** We could reduce the FFTLimit to only 512k, and that would drop the
** memory by 4 megs for the FFT, and increase it by SIZE * 2 * 2 for
** the multiplication product work space, for a total size of 10 megs.
**
** FFTLimit of 256k would consume a total of 8 megs.
** FFTLimit of 128k would consume a total of 7 megs.
**
** By reducing the size of the FFTLimit, you could reduce the memory
** used so it would fit in an 8meg machine.  For a 4meg computer,
** you are going to have to use a virtual memory compiler or OS if you
** actually want one million digits of pi.  On my computer, with an
** earlier version of the program, I was able to get 1 million digits
** in slightly under 3 hours.  But, using only 4 megs and virtual
** memory, and a reduced FFTLimit, it took about 17 hours!!  The
** current version of the program is *substantially* faster (about one
** hour instead of 3 hours), but the major speed different would
** still exist.  I haven't retimed it because I don't really care
** what the virtual memory run time is.  I'm just pointing out that there
** is a substantial speed penalty if you use it, and that if you do
** indeed need to use it, you can.  Ohh, one last note. If you do use
** virtual memory (ie: disk), I'd really recommend not using any disk
** compression (stacker, doublespace, drivespace, etc.) where the
** swap file is stored.  It wouldn't hurt anything, but it sure would
** be a waste of time.
**
** You could also disable the FFT entirely, by setting FFTLimit to 0,
** but the performance penalty would be so bad that there isn't any
** point in doing so.
**
** Timing results in seconds:
** Machine: Cyrix 486/66, 36megs, FFTLimit of 1048576 256k L2 cache
** DJGPP 2.7.2.1 with the options of -O3 -m486 -ffast-math
**
**     digits    time   Kanada's Super-Pi
**     4,096        6
**     8,192       14
**    16,384       31         50
**    32,768       72        107
**    65,536      166        236
**   131,072      384        550
**   262,144      872       1242
**   524,288     1966       2769
** 1,048,576     4399       6073
**
** The calculated digits have been verified against Gutenberg's
** pimil10.txt and it consistently has only the last 21-23 digits
** wrong.
**
** You can also see that my program is substantially faster than
** Kanada's famous SuperPi program.
**
** The previous version of my pi program had major L2 cache thrashing,
** with the result that I got faster pi runs when I had my L2 cache OFF.
**
** This version attempts to overcome that problem by using a recursive
** FFT to break the large FFT into chunks small enough to fit into the
** L2 cache.  That's controlled by the line in RFFT_T that compares
** it against 8192 (8192*16 bytes=128k).  128k was chosen because my
** cache is 256k and that lets me use the other for code or whatever.
** Also, 486's only have enough TLB entries to directly access 128k
** bytes without having to fetch an entry from main memory.  If you don't
** have a L2 cache, you might want to disable the recursion by changing
** the 8192 to FFTLimit.  If you are running a system that can better
** handle cache misses, you might want to increase that number to 16k
** or something.
**
** As a comparison against other systems, with a slightly earlier version
** of the program, a Pentium 200 was able to do 128k digits in 56 seconds
** and 256k digits in 133 seconds.
**
** Additional Notes:
**
** 1) The pi itself will be printed to standard output (stdout).  The
**    'running' information that it prints during the calculation is
**    printed to the standard error output.  The division allows you
**    to redirect stdout to a file and still see on the screen that
**    the program is still running.  MS-DOS, where this program was
**    developed, can only redirect stdout, so pi has to go there,
**    and I didn't want to combine it and the running information.
**
** 2) This program is really designed for large number of digits of
**    pi.  I mean tens of thousands to millions.  It'll work for fewer
**    digits, but if you are just wanting a few, you may be better off
**    using some other program.
**
** 3) It is strongly recommend that you use a FPU with this program.
**    A floating point package would work, but it sure would slow the
**    program down.  Even with an emulator, the program would still be
**    better for millions of digits of pi than an arctangent program.
**
**
** Thanks to the following people for testing, providing suggestions,
** code, timing results, etc.
**
** Vinhhao Nguyen       Andrew Frank        Cliff Rhodes
** Jerry Coffin         David Bailey        Jason Stratos Papadopoulos
*/
